@@ -386,9 +386,11 @@ handle_transfer_ok (const GnomeVFSXferProgressInfo *progress_info,
 	if (transfer_info->cancelled
 		&& progress_info->phase != GNOME_VFS_XFER_PHASE_COMPLETED) {
 		/* If cancelled, delete any partially copied files that are laying
-		 * around and return.
+		 * around and return. Don't delete the source though..
 		 */
 		if (progress_info->target_name != NULL
+		    && progress_info->source_name != NULL
+		    && strcmp (progress_info->source_name, progress_info->target_name) != 0
 		    && progress_info->bytes_total != progress_info->bytes_copied) {
 			GList *delete_me;
 
@@ -1054,7 +1056,8 @@ handle_transfer_overwrite (const GnomeVFSXferProgressInfo *progress_info,
 			(parent_for_error_dialog (transfer_info), TRUE, text, 
 			 _("Conflict while copying"),
 			 _("Replace"), _("Skip"), NULL);
-			 
+		g_free (text);	 
+
 		nautilus_file_operations_progress_resume_timeout (transfer_info->progress_dialog);
 					 
 		switch (result) {
@@ -1071,6 +1074,7 @@ handle_transfer_overwrite (const GnomeVFSXferProgressInfo *progress_info,
 			(parent_for_error_dialog (transfer_info), TRUE, text, 
 			 _("Conflict while copying"),
 			 _("Replace All"), _("Replace"), _("Skip"), NULL);
+		g_free (text);
 
 		nautilus_file_operations_progress_resume_timeout (transfer_info->progress_dialog);
 
