@@ -97,6 +97,8 @@ struct FMDirectoryViewClass {
 	 */
 	void 	(* end_file_changes)    (FMDirectoryView *view);
 	
+	void    (* flush_added_files)	 (FMDirectoryView *view);
+	
 	/* The 'begin_loading' signal is emitted before any of the contents
 	 * of a directory are added to the view. It can be replaced by a 
 	 * subclass to do any necessary preparation to start dealing with a
@@ -261,8 +263,10 @@ struct FMDirectoryViewClass {
 	 */
 	gboolean (* accepts_dragged_files)	(FMDirectoryView *view);
 
-	void	 (* start_renaming_item)        (FMDirectoryView *view,
-					  	 const char *uri);
+	gboolean (* can_rename_file)            (FMDirectoryView *view,
+						 NautilusFile *file);
+	void	 (* start_renaming_file)        (FMDirectoryView *view,
+					  	 NautilusFile *file);
 
 	gboolean (* file_still_belongs)		(FMDirectoryView *view,
 						 NautilusFile	 *file);
@@ -275,20 +279,20 @@ struct FMDirectoryViewClass {
 	void	(* embedded_text_policy_changed)   (FMDirectoryView *view);
 	void	(* image_display_policy_changed)   (FMDirectoryView *view);
 	void	(* click_policy_changed)	   (FMDirectoryView *view);
-	void	(* smooth_graphics_mode_changed)   (FMDirectoryView *view);
 	void	(* sort_directories_first_changed) (FMDirectoryView *view);
 
 	void	(* emblems_changed)                (FMDirectoryView *view);
 };
 
-/* GtkObject support */
-GtkType             fm_directory_view_get_type                         (void);
+/* GObject support */
+GType               fm_directory_view_get_type                         (void);
 
 /* Component embedding support */
 NautilusView *      fm_directory_view_get_nautilus_view                (FMDirectoryView  *view);
 
 /* Functions callable from the user interface and elsewhere. */
 char *              fm_directory_view_get_uri                          (FMDirectoryView  *view);
+char *              fm_directory_view_get_backing_uri                  (FMDirectoryView  *view);
 gboolean            fm_directory_view_can_accept_item                  (NautilusFile     *target_item,
 									const char       *item_uri,
 									FMDirectoryView  *view);
@@ -309,7 +313,7 @@ void                fm_directory_view_reset_to_defaults                (FMDirect
 void                fm_directory_view_select_all                       (FMDirectoryView  *view);
 void                fm_directory_view_set_selection                    (FMDirectoryView  *view,
 									GList            *selection);
-GArray *            fm_directory_get_selected_icon_locations           (FMDirectoryView  *view);
+GArray *            fm_directory_view_get_selected_icon_locations      (FMDirectoryView  *view);
 void                fm_directory_view_reveal_selection                 (FMDirectoryView  *view);
 gboolean            fm_directory_view_is_empty                         (FMDirectoryView  *view);
 gboolean            fm_directory_view_is_read_only                     (FMDirectoryView  *view);
