@@ -177,7 +177,7 @@ fm_list_view_did_not_drag (FMListView *view,
 
 	if (gtk_tree_view_get_path_at_pos (tree_view, event->x, event->y,
 					   &path, NULL, NULL, NULL)) {
-		if(event->button == 1
+		if((event->button == 1 || event->button == 2)
 		   && (click_policy_auto_value == NAUTILUS_CLICK_POLICY_DOUBLE)
 		   && gtk_tree_selection_path_is_selected (selection, path)
 		   && !button_event_modifies_selection (event)) {
@@ -274,7 +274,7 @@ stop_drag_check (FMListView *view)
 }
 
 static GdkPixbuf *
-get_drag_pixbuf (FMListView *view, int *offset_x, int *offset_y)
+get_drag_pixbuf (FMListView *view)
 {
 	GtkTreeModel *model;
 	GtkTreePath *path;
@@ -299,8 +299,6 @@ get_drag_pixbuf (FMListView *view, int *offset_x, int *offset_y)
 					     path, 
 					     view->details->file_name_column, 
 					     &cell_area);
-		*offset_x = view->details->drag_x - cell_area.x;
-		*offset_y = view->details->drag_y - cell_area.y;
 
 		gtk_tree_path_free (path);
 	}
@@ -317,7 +315,6 @@ motion_notify_callback (GtkWidget *widget,
 	GdkDragContext *context;
 	GList *ref_list;
 	GdkPixbuf *pixbuf;	
-	int offset_x, offset_y;
 	
 	view = FM_LIST_VIEW (callback_data);
 	
@@ -347,12 +344,11 @@ motion_notify_callback (GtkWidget *widget,
 						ref_list,
 						(GDestroyNotify)ref_list_free);
 
-			pixbuf = get_drag_pixbuf (view, &offset_x, &offset_y);
+			pixbuf = get_drag_pixbuf (view);
 			if (pixbuf) {
 				gtk_drag_set_icon_pixbuf (context,
 							  pixbuf,
-							  offset_x, 
-							  offset_y);
+							  0, 0);
 				g_object_unref (pixbuf);
 			} else {
 				gtk_drag_set_icon_default (context);
@@ -450,16 +446,20 @@ button_press_callback (GtkWidget *widget, GdkEventButton *event, gpointer callba
 		 * want that; we want the right click menu or single
 		 * click to apply to everything that's currently selected. */
 		
-		if (event->button == 3 && gtk_tree_selection_path_is_selected (selection, path)) {	
+		if (event->button == 3 && gtk_tree_selection_path_is_selected (selection, path)) {
 			call_parent = FALSE;
 		} 
 
-		if(!button_event_modifies_selection (event) && event->button == 1 && gtk_tree_selection_path_is_selected (selection, path)) {
+		if(!button_event_modifies_selection (event) &&
+		   (event->button == 1 || event->button == 2) &&
+		   gtk_tree_selection_path_is_selected (selection, path)) {
 			call_parent = FALSE;
 		}
 
 		if (call_parent) {
 			tree_view_class->button_press_event (widget, event);
+		} else if (gtk_tree_selection_path_is_selected (selection, path)) {
+			gtk_widget_grab_focus(widget);
 		}
 
 		if (event->button == 1 || event->button == 2) {
@@ -531,11 +531,14 @@ key_press_callback (GtkWidget *widget, GdkEventKey *event, gpointer callback_dat
 		}
 		break;
 	case GDK_space:
-		if ((event->state & GDK_CONTROL_MASK) == 0) {
-			activate_selected_items (FM_LIST_VIEW (view));
-			return TRUE;
+		if (event->state & GDK_CONTROL_MASK) {
+			return FALSE;
 		}
-		break;
+		if (!GTK_WIDGET_HAS_FOCUS (GTK_WIDGET (FM_LIST_VIEW (view)->details->tree_view))) {
+			return FALSE;
+		}
+		activate_selected_items (FM_LIST_VIEW (view));
+		return TRUE;
 	case GDK_Return:
 	case GDK_KP_Enter:
 		activate_selected_items (FM_LIST_VIEW (view));
@@ -1333,7 +1336,13 @@ list_view_get_first_visible_file_callback (NautilusScrollPositionable *positiona
 				    FM_LIST_MODEL_FILE_COLUMN, &file,
 				    -1);
 		if (file) {
-			return nautilus_file_get_uri (file);
+			char *uri;
+			
+			uri = nautilus_file_get_uri (file);
+			
+			nautilus_file_unref (file);
+			
+			return uri;
 		}
 	}
 	
