@@ -92,7 +92,7 @@ eel_gtk_main_quit_all (void)
 	 * This idler will be run by the current event loop, killing it, and then
 	 * by the next event loop, ...
 	 */
-	gtk_idle_add (quit_if_in_main_loop, NULL);
+	g_idle_add (quit_if_in_main_loop, NULL);
 }
 
 static void
@@ -141,25 +141,36 @@ main (int argc, char *argv[])
 	char **argv_copy;
 	GnomeProgram *program;
 	GValue context_as_value = { 0 };
+	int i;
 
 	struct poptOption options[] = {
 #ifndef NAUTILUS_OMIT_SELF_CHECK
-		{ "check", 'c', POPT_ARG_NONE, &perform_self_check, 0,
+		{ "check", 'c', POPT_ARG_NONE, NULL, 0,
 		  N_("Perform a quick set of self-check tests."), NULL },
 #endif
-		{ "geometry", 'g', POPT_ARG_STRING, &geometry, 0,
+		{ "geometry", 'g', POPT_ARG_STRING, NULL, 0,
 		  N_("Create the initial window with the given geometry."), N_("GEOMETRY") },
-		{ "no-default-window", 'n', POPT_ARG_NONE, &no_default_window, 0,
+		{ "no-default-window", 'n', POPT_ARG_NONE, NULL, 0,
 		  N_("Only create windows for explicitly specified URIs."), NULL },
-		{ "no-desktop", '\0', POPT_ARG_NONE, &no_desktop, 0,
+		{ "no-desktop", '\0', POPT_ARG_NONE, NULL, 0,
 		  N_("Do not manage the desktop (ignore the preference set in the preferences dialog)."), NULL },
-		{ "quit", 'q', POPT_ARG_NONE, &kill_shell, 0,
+		{ "quit", 'q', POPT_ARG_NONE, NULL, 0,
 		  N_("Quit Nautilus."), NULL },
-		{ "restart", '\0', POPT_ARG_NONE | POPT_ARGFLAG_DOC_HIDDEN, &restart_shell, 0,
+		{ "restart", '\0', POPT_ARG_NONE | POPT_ARGFLAG_DOC_HIDDEN, NULL, 0,
 		  N_("Restart Nautilus."), NULL },
 		{ NULL, '\0', 0, NULL, 0, NULL, NULL }
 	};
 
+	i = 0;
+#ifndef NAUTILUS_OMIT_SELF_CHECK
+	options[i++].arg = &perform_self_check;
+#endif
+	options[i++].arg = &geometry;
+	options[i++].arg = &no_default_window;
+	options[i++].arg = &no_desktop;
+	options[i++].arg = &kill_shell;
+	options[i++].arg = &restart_shell;
+
 	if (g_getenv ("NAUTILUS_DEBUG") != NULL) {
 		eel_make_warnings_and_criticals_stop_in_debugger ();
 	}
@@ -177,13 +188,20 @@ main (int argc, char *argv[])
 	perform_self_check = FALSE;
 	restart_shell = FALSE;
 
+	g_set_application_name (_("File Manager"));
+	
 	program = gnome_program_init ("nautilus", VERSION,
 				      LIBGNOMEUI_MODULE, argc, argv,
 				      GNOME_PROGRAM_STANDARD_PROPERTIES,
 				      GNOME_PARAM_POPT_TABLE, options,
 				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Nautilus"),
 				      NULL);
-
+	
+	/* Need to set this to the canonical DISPLAY value, since
+	   thats where we're registering per-display components */
+	bonobo_activation_set_activation_env_value ("DISPLAY",
+						    gdk_display_get_name (gdk_display_get_default()));
+	
 	g_object_get_property (G_OBJECT (program),
 			       GNOME_PARAM_POPT_CONTEXT,
 			       g_value_init (&context_as_value, G_TYPE_POINTER));
@@ -234,6 +252,8 @@ main (int argc, char *argv[])
 	
 	bonobo_activate (); /* do now since we need it before main loop */
 
+	application = NULL;
+ 
 	/* Do either the self-check or the real work. */
 	if (perform_self_check) {
 #ifndef NAUTILUS_OMIT_SELF_CHECK
@@ -261,12 +281,19 @@ main (int argc, char *argv[])
 		if (is_event_loop_needed ()) {
 			gtk_main ();
 		}
-		bonobo_object_unref (application);
 	}
 
 	poptFreeContext (popt_context);
 
 	gnome_vfs_shutdown ();
+
+	/* This has to be done after gnome_vfs_shutdown, because shutdown
+	 * can call pending completion callbacks which reference application.
+	 */
+	if (application != NULL) {
+		bonobo_object_unref (application);
+	}
+
 	eel_debug_shut_down ();
 	bonobo_ui_debug_shutdown ();
 
