@@ -30,11 +30,15 @@
 
 #include "nautilus-applicable-views.h"
 #include "nautilus-application.h"
+#include "nautilus-information-panel.h"
 #include "nautilus-location-bar.h"
 #include "nautilus-main.h"
 #include "nautilus-window-private.h"
 #include "nautilus-zoom-control.h"
+#include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-ui-util.h>
+#include <bonobo/bonobo-property-bag-client.h>
+#include <eel/eel-accessibility.h>
 #include <eel/eel-debug.h>
 #include <eel/eel-gdk-extensions.h>
 #include <eel/eel-glib-extensions.h>
@@ -44,6 +48,7 @@
 #include <eel/eel-vfs-extensions.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
+#include <gdk/gdkx.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-dialog-util.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
@@ -58,6 +63,7 @@
 #include <libnautilus-private/nautilus-mime-actions.h>
 #include <libnautilus-private/nautilus-monitor.h>
 #include <libnautilus-private/nautilus-search-uri.h>
+#include <libnautilus-private/nautilus-theme.h>
 
 /* FIXME bugzilla.gnome.org 41243: 
  * We should use inheritance instead of these special cases
@@ -102,7 +108,8 @@ static void disconnect_view        (NautilusWindow             *window,
 static void begin_location_change  (NautilusWindow             *window,
                                     const char                 *location,
                                     NautilusLocationChangeType  type,
-                                    guint                       distance);
+                                    guint                       distance,
+                                    const char                 *scroll_pos);
 static void free_location_change   (NautilusWindow             *window);
 static void end_location_change    (NautilusWindow             *window);
 static void cancel_location_change (NautilusWindow             *window);
@@ -179,7 +186,11 @@ compute_title (NautilusWindow *window)
         char *title;
 
 	title = NULL;
-        if (window->new_content_view != NULL) {
+	
+	if (NAUTILUS_IS_DESKTOP_WINDOW (window)) {
+		/* Special Desktop window title (displayed in the Ctrl-Alt-Tab window) */
+		title = g_strdup(_("Desktop"));
+	} else if (window->new_content_view != NULL) {
                 title = nautilus_view_frame_get_title (window->new_content_view);
         } else if (window->content_view != NULL) {
                 title = nautilus_view_frame_get_title (window->content_view);
@@ -224,12 +235,12 @@ update_title (NautilusWindow *window)
                 gtk_window_set_title (GTK_WINDOW (window), window_title);
                 g_free (window_title);
         }
-
-	if (window->sidebar != NULL) {
-        	nautilus_sidebar_set_title (window->sidebar, title);
+	if (window->information_panel) {
+        	nautilus_information_panel_set_title 
+                        (window->information_panel, title);
 	}
         
-        if (title [0] != '\0' &&
+        if (title [0] != '\0' && window->current_location_bookmark &&
             nautilus_bookmark_set_name (window->current_location_bookmark, title)) {
                 /* Name of item in history list changed, tell listeners. */
                 nautilus_send_history_list_changed ();
@@ -252,6 +263,42 @@ update_title (NautilusWindow *window)
         g_free (title);
 }
 
+/* nautilus_window_update_icon:
+ * 
+ * Update the non-NautilusViewFrame objects that use the location's user-displayable
+ * icon in some way. Called when the location or icon-theme has changed.
+ * @window: The NautilusWindow in question.
+ * 
+ */
+void
+nautilus_window_update_icon (NautilusWindow *window)
+{
+	char *path;
+	GdkPixbuf *pixbuf;
+
+	pixbuf = NULL;
+	
+	/* Desktop window special icon */
+	if (NAUTILUS_IS_DESKTOP_WINDOW (window)) {
+		path = nautilus_pixmap_file ("nautilus-desktop.png");
+
+		if (path != NULL) {
+			pixbuf = gdk_pixbuf_new_from_file (path, NULL);
+			
+			g_free (path);
+		}
+	} else {
+		pixbuf = nautilus_icon_factory_get_pixbuf_for_file (window->details->viewed_file,
+								    "open",
+								    NAUTILUS_ICON_SIZE_STANDARD);
+	}
+
+	if (pixbuf != NULL) {
+		gtk_window_set_icon (GTK_WINDOW (window), pixbuf);
+		g_object_unref (pixbuf);
+	}
+}
+
 /* set_displayed_location:
  * 
  * Update the non-NautilusViewFrame objects that use the location's user-displayable
@@ -284,6 +331,7 @@ set_displayed_location (NautilusWindow *window, const char *location)
         }
 
         update_title (window);
+	nautilus_window_update_icon (window);
 }
 
 static void
@@ -423,6 +471,16 @@ update_up_button (NautilusWindow *window)
         nautilus_window_allow_up (window, allowed);
 }
 
+static void
+update_burn_cd_items (NautilusWindow *window)
+{
+        gboolean show_burn_cd;
+        
+        show_burn_cd = eel_istr_has_prefix (window->details->location, "burn:");
+
+        nautilus_window_allow_burn_cd (window, show_burn_cd);
+}
+
 static void
 viewed_file_changed_callback (NautilusFile *file,
                               NautilusWindow *window)
@@ -495,6 +553,7 @@ viewed_file_changed_callback (NautilusFile *file,
                 }
 
                 update_title (window);
+		nautilus_window_update_icon (window);
         }
 }
 
@@ -563,7 +622,7 @@ update_for_new_location (NautilusWindow *window)
         file = nautilus_file_get (window->details->location);
         nautilus_window_set_viewed_file (window, file);
         window->details->viewed_file_seen = !nautilus_file_is_not_yet_confirmed (file);
-        nautilus_file_monitor_add (file, &window->details->viewed_file, NULL);
+        nautilus_file_monitor_add (file, &window->details->viewed_file, 0);
         g_signal_connect_object (file, "changed",
                                  G_CALLBACK (viewed_file_changed_callback), window, 0);
         nautilus_file_unref (file);
@@ -571,6 +630,9 @@ update_for_new_location (NautilusWindow *window)
         /* Check if we can go up. */
         update_up_button (window);
         
+        /* Check if we can go up. */
+        update_burn_cd_items (window);
+        
         /* Set up the content view menu for this new location. */
         nautilus_window_load_view_as_menus (window);
         
@@ -582,15 +644,15 @@ update_for_new_location (NautilusWindow *window)
         nautilus_navigation_bar_set_location (NAUTILUS_NAVIGATION_BAR (window->navigation_bar),
                                               window->details->location);
         
-        /* Notify the sidebar of the location change. */
+        /* Notify the information panel of the location change. */
         /* FIXME bugzilla.gnome.org 40211:
          * Eventually, this will not be necessary when we restructure the 
          * sidebar itself to be a NautilusViewFrame.
          */
-	if (window->sidebar != NULL) {
-		nautilus_sidebar_set_uri (window->sidebar,
-					  window->details->location,
-					  window->details->title);
+	if (window->information_panel) {
+		nautilus_information_panel_set_uri (window->information_panel,
+                                                    window->details->location,
+                                                    window->details->title);
 	}
 }
 
@@ -608,12 +670,6 @@ ref_now_unref_at_idle_time (GObject *object)
         g_idle_add (unref_callback, object);
 }
 
-static gboolean
-use_saved_window_positions (void)
-{
-        return eel_preferences_get_boolean (NAUTILUS_PREFERENCES_WINDOW_ALWAYS_NEW);
-}
-
 /* This is called when we have decided we can actually change to the new view/location situation. */
 static void
 location_has_really_changed (NautilusWindow *window)
@@ -652,14 +708,14 @@ location_has_really_changed (NautilusWindow *window)
         free_location_change (window);
 
         update_title (window);
+	nautilus_window_update_icon (window);
 
         /* The whole window has been finished. Now show it, unless
          * we're still waiting for the saved positions from the
          * metadata. Then tell the callback it needs to show the
          * window
          */
-        if (!use_saved_window_positions () ||
-            window->show_state == NAUTILUS_WINDOW_POSITION_SET) {
+        if (window->show_state == NAUTILUS_WINDOW_POSITION_SET) {
                 gtk_widget_show (GTK_WIDGET (window));
         } else {
                 window->show_state = NAUTILUS_WINDOW_SHOULD_SHOW;
@@ -698,14 +754,20 @@ open_location (NautilusWindow *window,
         }
 
         if (create_new_window) {
-                target_window = nautilus_application_create_window (window->application);
-        }
+                target_window = nautilus_application_create_window (
+						window->application,
+						gtk_window_get_screen (GTK_WINDOW (window)));
+	}
 
 	eel_g_list_free_deep (target_window->details->pending_selection);
         target_window->details->pending_selection = eel_g_str_list_copy (new_selection);
 
+        if (!eel_is_valid_uri (location))
+                g_warning ("Possibly invalid new URI '%s'\n"
+                           "This can cause subtle evils like #48423", location);
+
         begin_location_change (target_window, location,
-                               NAUTILUS_LOCATION_CHANGE_STANDARD, 0);
+                               NAUTILUS_LOCATION_CHANGE_STANDARD, 0, NULL);
 }
 
 void
@@ -976,6 +1038,10 @@ load_content_view (NautilusWindow *window,
                 /* create a new content view */
                 view = nautilus_view_frame_new (window->details->ui_container,
                                                 window->application->undo_manager);
+
+                eel_accessibility_set_name (view, _("Content View"));
+                eel_accessibility_set_description (view, _("View of the current file or folder"));
+                
                 window->new_content_view = view;
                 g_object_ref (view);
                 gtk_object_sink (GTK_OBJECT (view));
@@ -1033,9 +1099,6 @@ handle_view_failure (NautilusWindow *window,
 	if (view_frame_is_sidebar_panel (view)) {
                 report_sidebar_panel_failure_to_user (window, view);
 		current_iid = nautilus_view_frame_get_view_iid (view);
-		if (window->sidebar != NULL) {
-			nautilus_sidebar_hide_active_panel_if_matches (window->sidebar, current_iid);
-		}
 		disconnect_and_destroy_sidebar_panel (window, view);
 	} else {
 	        if (view == window->content_view) {
@@ -1066,6 +1129,10 @@ free_location_change (NautilusWindow *window)
         g_free (window->details->pending_location);
         window->details->pending_location = NULL;
 
+        /* Don't free pending_scroll_to, since thats needed until
+         * the load_complete callback.
+         */
+
         if (window->details->determine_view_handle != NULL) {
                 nautilus_determine_initial_view_cancel (window->details->determine_view_handle);
                 window->details->determine_view_handle = NULL;
@@ -1085,6 +1152,13 @@ static void
 end_location_change (NautilusWindow *window)
 {
         nautilus_window_allow_stop (window, FALSE);
+
+        /* Now we can free pending_scroll_to, since the load_complete
+         * callback already has been emitted.
+         */
+        g_free (window->details->pending_scroll_to);
+        window->details->pending_scroll_to = NULL;
+
         free_location_change (window);
 }
 
@@ -1111,27 +1185,61 @@ cancel_location_change (NautilusWindow *window)
         end_location_change (window);
 }
 
+
+static gboolean
+pending_location_already_showing (NautilusWindow *window)
+{
+	char *temp;
+	char *location;
+	GList *list, *item;
+	
+	temp = window->details->pending_location;
+	list = nautilus_application_get_window_list ();
+	for (item = list; item != NULL; item = item->next) {
+		location = nautilus_window_get_location (NAUTILUS_WINDOW (item->data));
+
+		if (!NAUTILUS_IS_DESKTOP_WINDOW (item->data)
+		    && location != NULL
+		    && item->data != window
+		    && !strcmp (temp, location)) {
+		    	g_free (location);
+			return TRUE;
+		}
+		
+		g_free (location);
+	}
+	
+	return FALSE;
+}
+
+
 static void
 position_and_show_window_callback (NautilusFile *file,
                        		   gpointer callback_data)
 {
 	NautilusWindow *window;
 	char *geometry_string;
-        
+   
 	window = NAUTILUS_WINDOW (callback_data);
 
-	if (use_saved_window_positions ()) {
-		geometry_string = nautilus_file_get_metadata 
+        if (!NAUTILUS_IS_DESKTOP_WINDOW (window)) {
+                /* load the saved window geometry */
+                geometry_string = nautilus_file_get_metadata 
 			(file, NAUTILUS_METADATA_KEY_WINDOW_GEOMETRY, NULL);
-		if (geometry_string != NULL) {
-			eel_gtk_window_set_initial_geometry_from_string 
-				(GTK_WINDOW (window), 
-				 geometry_string,
-				 NAUTILUS_WINDOW_MIN_WIDTH, 
-				 NAUTILUS_WINDOW_MIN_HEIGHT);
-		}
-		g_free (geometry_string);
-	}
+                if (geometry_string != NULL) {
+			/* Ignore saved window position if a window with the same
+			 * location is already showing. That way the two windows
+			 * wont appear at the exact same location on the screen.
+			 */
+                        eel_gtk_window_set_initial_geometry_from_string 
+                                (GTK_WINDOW (window), 
+                                 geometry_string,
+                                 NAUTILUS_WINDOW_MIN_WIDTH, 
+                                 NAUTILUS_WINDOW_MIN_HEIGHT,
+				 pending_location_already_showing (window));
+                }
+                g_free (geometry_string);
+        }
 
         /* If we finished constructing the window by now we need
          * to show the window here.
@@ -1170,7 +1278,7 @@ determined_initial_view_callback (NautilusDetermineViewHandle *handle,
         char *home_uri;
         const char *location;
 	GtkDialog *dialog;
-        GList *attributes;
+        NautilusFileAttributes attributes;
         GnomeVFSURI *vfs_uri;
        
         window = NAUTILUS_WINDOW (data);
@@ -1186,22 +1294,19 @@ determined_initial_view_callback (NautilusDetermineViewHandle *handle,
 		 * windows), position and show it only after we've got the
 		 * metadata (since position info is stored there).
 		 */
-                if (use_saved_window_positions ()) {
-                        window->show_state = NAUTILUS_WINDOW_NOT_SHOWN;
-                        if (!GTK_WIDGET_VISIBLE (window)) {
-                                file = nautilus_file_get (location);
+		window->show_state = NAUTILUS_WINDOW_NOT_SHOWN;
+		if (!GTK_WIDGET_VISIBLE (window)) {
+			file = nautilus_file_get (location);
                                 
-                                attributes = g_list_prepend (NULL, NAUTILUS_FILE_ATTRIBUTE_METADATA);
-                                nautilus_file_call_when_ready (file,
-                                                               attributes,
-                                                               position_and_show_window_callback,
-                                                               window);
-                                g_list_free (attributes);
-                        }
-                }
+			attributes = NAUTILUS_FILE_ATTRIBUTE_METADATA;
+			nautilus_file_call_when_ready (file,
+                                                       attributes,
+                                                       position_and_show_window_callback,
+                                                       window);
+		}
 
-                load_content_view (window, initial_view);
-                return;
+		load_content_view (window, initial_view);
+		return;
         }
         
         /* Some sort of failure occurred. How 'bout we tell the user? */
@@ -1290,11 +1395,18 @@ determined_initial_view_callback (NautilusDetermineViewHandle *handle,
 		break;
 
 	case NAUTILUS_DETERMINE_VIEW_HOST_HAS_NO_ADDRESS:
-                error_message = g_strdup_printf (_("Couldn't display \"%s\", because the host name was empty. "
+                error_message = g_strdup_printf (_("Couldn't display \"%s\". "
                 				   "Check that your proxy settings are correct."),
                                                  uri_for_display);
 		break;
 
+        case NAUTILUS_DETERMINE_VIEW_NO_MASTER_BROWSER:
+                error_message = g_strdup_printf
+                        (_("Couldn't display \"%s\", because Nautilus cannot contact the SMB master browser.\n"
+                           "Check that an SMB server is running in the local network."),
+                         uri_for_display);
+                break;
+
 	case NAUTILUS_DETERMINE_VIEW_SERVICE_NOT_AVAILABLE:
 		if (nautilus_is_search_uri (location)) {
 			/* FIXME bugzilla.gnome.org 42458: Need to give
@@ -1308,17 +1420,7 @@ determined_initial_view_callback (NautilusDetermineViewHandle *handle,
                                    "don't have an index, that the Medusa indexer is running."));
 			dialog_title = g_strdup (_("Searching Unavailable"));
 			break;
-		} else {
-			/* This is a special case for the smb: module */
-			if (eel_istr_has_prefix(location, "smb:") == 0)
-			{
-				error_message = g_strdup_printf
-					(_("Couldn't display \"%s\", because Nautilus cannot contact the SMB master browser.\n"
-					"Check that an SMB server is running in the local network."),
-					uri_for_display);
-				break;
-			}
-		}
+		} 
 		/* else fall through */
         default:
                 error_message = g_strdup_printf (_("Nautilus cannot display \"%s\"."),
@@ -1343,7 +1445,11 @@ determined_initial_view_callback (NautilusDetermineViewHandle *handle,
 			   better test for that */
 
                         if (!eel_uris_match (location, "file:///")) {
+#ifdef WEB_NAVIGATION_ENABLED
                                 home_uri = eel_preferences_get (NAUTILUS_PREFERENCES_HOME_URI);
+#else
+                                home_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
+#endif
                                 if (!eel_uris_match (home_uri, location)) {	
                                         nautilus_window_go_home (NAUTILUS_WINDOW (window));
                                 } else {
@@ -1382,16 +1488,19 @@ determined_initial_view_callback (NautilusDetermineViewHandle *handle,
  * @type: Which type of location change is this? Standard, back, forward, or reload?
  * @distance: If type is back or forward, the index into the back or forward chain. If
  * type is standard or reload, this is ignored, and must be 0.
+ * @scroll_pos: The file to scroll to when the location is loaded.
  */
 static void
 begin_location_change (NautilusWindow *window,
                        const char *location,
                        NautilusLocationChangeType type,
-                       guint distance)
+                       guint distance,
+                       const char *scroll_pos)
 {
         NautilusDirectory *directory;
         NautilusFile *file;
 	gboolean force_reload;
+        char *current_pos;
 
         g_assert (NAUTILUS_IS_WINDOW (window));
         g_assert (location != NULL);
@@ -1404,11 +1513,14 @@ begin_location_change (NautilusWindow *window,
         end_location_change (window);
         
         nautilus_window_allow_stop (window, TRUE);
+        nautilus_window_set_status (window, " ");
 
         window->details->pending_location = g_strdup (location);
         window->details->location_change_type = type;
         window->details->location_change_distance = distance;
-
+        
+        window->details->pending_scroll_to = g_strdup (scroll_pos);
+        
         directory = nautilus_directory_get (location);
 
 	/* The code to force a reload is here because if we do it
@@ -1431,6 +1543,14 @@ begin_location_change (NautilusWindow *window,
 	}
 
         nautilus_directory_unref (directory);
+
+        /* Set current_bookmark scroll pos */
+        if (window->current_location_bookmark != NULL &&
+            window->content_view != NULL) {
+                current_pos = nautilus_view_frame_get_first_visible_file (window->content_view);
+                nautilus_bookmark_set_scroll_pos (window->current_location_bookmark, current_pos);
+                g_free (current_pos);
+        }
         
         window->details->determine_view_handle = nautilus_determine_initial_view
                 (location, determined_initial_view_callback, window);
@@ -1482,18 +1602,24 @@ nautilus_window_set_content_view (NautilusWindow *window,
         g_return_if_fail (window->details->location != NULL);
 	g_return_if_fail (id != NULL);
 
-        end_location_change (window);
-
         if (nautilus_window_content_view_matches_iid (window, id->iid)) {
         	return;
         }
 
+        end_location_change (window);
+
 	file = nautilus_file_get (window->details->location);
         nautilus_mime_set_default_component_for_file
 		(file, id->iid);
         nautilus_file_unref (file);
         
         nautilus_window_allow_stop (window, TRUE);
+
+        if (window->details->selection == NULL) {
+                /* If there is no selection, queue a scroll to the same icon that
+                 * is currently visible */
+                window->details->pending_scroll_to = nautilus_view_frame_get_first_visible_file (window->content_view);
+        }
         
         load_content_view (window, id);
 }
@@ -1532,9 +1658,6 @@ nautilus_window_set_sidebar_panels (NautilusWindow *window,
 						 compare_view_identifier_with_iid);
 		if (found_node == NULL) {
 			current_iid = nautilus_view_frame_get_view_iid (sidebar_panel);
-			if (window->sidebar != NULL) {
-				nautilus_sidebar_hide_active_panel_if_matches (window->sidebar, current_iid);
-			}
 			disconnect_and_destroy_sidebar_panel (window, sidebar_panel);
 		} else {
                         identifier = (NautilusViewIdentifier *) found_node->data;
@@ -1560,11 +1683,17 @@ nautilus_window_set_sidebar_panels (NautilusWindow *window,
                 /* Create and load the panel. */
 		sidebar_panel = nautilus_view_frame_new (window->details->ui_container,
                                                          window->application->undo_manager);
+                
+                eel_accessibility_set_name (sidebar_panel, _("Side Pane"));
+                eel_accessibility_set_description
+                        (sidebar_panel, _("Contains a side pane view"));
+                
+
 		nautilus_view_frame_set_label (sidebar_panel, identifier->name);
 		set_view_frame_info (sidebar_panel, TRUE, identifier);
 		connect_view (window, sidebar_panel);
-		nautilus_window_add_sidebar_panel (window, sidebar_panel);
 		nautilus_view_frame_load_view (sidebar_panel, identifier->iid);
+		nautilus_window_add_sidebar_panel (window, sidebar_panel);
                 gtk_object_sink (GTK_OBJECT (sidebar_panel));
 	}
 
@@ -1788,6 +1917,10 @@ load_complete_callback (NautilusViewFrame *view,
          */
 
         if (view == window->content_view) {
+                if (window->details->pending_scroll_to != NULL) {
+                        nautilus_view_frame_scroll_to_file (window->content_view,
+                                                            window->details->pending_scroll_to);
+                }
                 end_location_change (window);
         }
 }
@@ -1919,6 +2052,91 @@ title_changed_callback (NautilusViewFrame *view,
         g_assert (NAUTILUS_IS_WINDOW (window));
 
         update_title (window);
+	nautilus_window_update_icon (window);
+}
+
+static void
+set_side_panel_image (NautilusWindow *window,
+                      NautilusViewFrame *side_panel,
+                      const char *image_name)
+{
+        GdkPixbuf *pixbuf;
+        char *image_path;
+
+        pixbuf = NULL;
+        
+        if (image_name && image_name[0]) {
+                image_path = nautilus_theme_get_image_path (image_name);
+                if (image_path) {
+                        pixbuf = gdk_pixbuf_new_from_file (image_path, NULL);
+                        g_free (image_path);
+                }
+        }
+
+        nautilus_side_pane_set_panel_image (window->sidebar,
+                                            GTK_WIDGET (side_panel),
+                                            pixbuf);
+        
+        if (pixbuf) {
+                g_object_unref (pixbuf);
+        }
+}
+
+static void
+side_panel_image_changed_callback (BonoboListener *listener,
+                                   const char *event_name,
+                                   const CORBA_any *arg,
+                                   CORBA_Environment *ev,
+                                   gpointer callback_data)
+{
+        NautilusViewFrame *side_panel;
+        NautilusWindow *window;
+
+        side_panel = NAUTILUS_VIEW_FRAME (callback_data);        
+        window = NAUTILUS_WINDOW (g_object_get_data (G_OBJECT (side_panel),
+                                                     "nautilus-window"));
+
+        set_side_panel_image (window, side_panel, BONOBO_ARG_GET_STRING (arg));
+}
+
+static void
+connect_side_panel (NautilusWindow *window,
+                    NautilusViewFrame *side_panel)
+{
+        Bonobo_Control control;
+        Bonobo_PropertyBag property_bag;
+        CORBA_Environment ev;
+        char *image_name;
+        
+        g_object_set_data (G_OBJECT (side_panel),
+                           "nautilus-window",
+                           window);
+        
+        control = nautilus_view_frame_get_control (side_panel);
+
+        if (control != CORBA_OBJECT_NIL) {
+                CORBA_exception_init (&ev);
+                property_bag = Bonobo_Control_getProperties (control, &ev);
+                if (property_bag != CORBA_OBJECT_NIL) {                        
+                        bonobo_event_source_client_add_listener 
+                                (property_bag,
+                                 side_panel_image_changed_callback,
+                                 "Bonobo/Property:change:tab_image",
+                                 NULL,
+                                 side_panel);
+                        
+                        /* Set the initial tab image */
+                        image_name = bonobo_property_bag_client_get_value_string
+                                (property_bag, 
+                                 "tab_image", 
+                                 NULL);
+                        set_side_panel_image (window, side_panel, image_name);
+                        g_free (image_name);
+                        
+                        bonobo_object_release_unref (property_bag, NULL);
+                }
+                CORBA_exception_free (&ev);
+        }
 }
 
 static void
@@ -1940,6 +2158,11 @@ view_loaded_callback (NautilusViewFrame *view,
                                                        window->details->selection);
                 }
         }
+
+        if (view_frame_is_sidebar_panel (view)) {
+                connect_side_panel (window, view);
+        }
+
         if (window->details->title != NULL) {
                 nautilus_view_frame_title_changed (view, window->details->title);
         }
@@ -2021,7 +2244,9 @@ nautilus_window_back_or_forward (NautilusWindow *window, gboolean back, guint di
 {
 	GList *list;
 	char *uri;
+        char *scroll_pos;
         guint len;
+        NautilusBookmark *bookmark;
 	
 	list = back ? window->back_list : window->forward_list;
 
@@ -2036,14 +2261,18 @@ nautilus_window_back_or_forward (NautilusWindow *window, gboolean back, guint di
         if (distance >= len)
                 distance = len - 1;
 
-	uri = nautilus_bookmark_get_uri (g_list_nth_data (list, distance));
+        bookmark = g_list_nth_data (list, distance);
+	uri = nautilus_bookmark_get_uri (bookmark);
+        scroll_pos = nautilus_bookmark_get_scroll_pos (bookmark);
 	begin_location_change
 		(window,
 		 uri,
 		 back ? NAUTILUS_LOCATION_CHANGE_BACK : NAUTILUS_LOCATION_CHANGE_FORWARD,
-		 distance);
+		 distance,
+                 scroll_pos);
 
 	g_free (uri);
+        g_free (scroll_pos);
 }
 
 /* reload the contents of the window */
@@ -2051,6 +2280,7 @@ void
 nautilus_window_reload (NautilusWindow *window)
 {
 	char *location;
+        char *current_pos;
 	
         g_return_if_fail (NAUTILUS_IS_WINDOW (window));
 
@@ -2058,8 +2288,10 @@ nautilus_window_reload (NautilusWindow *window)
 	 * of begin_location_change, so make a copy
 	 */
 	location = g_strdup (window->details->location);
+        current_pos = nautilus_view_frame_get_first_visible_file (window->content_view);
 	begin_location_change
 		(window, location,
-		 NAUTILUS_LOCATION_CHANGE_RELOAD, 0);
+		 NAUTILUS_LOCATION_CHANGE_RELOAD, 0, current_pos);
+        g_free (current_pos);
 	g_free (location);
 }
